"use strict";
/*!
    MIT License

    https://github.com/jonathandann/yalesyncalarm
    Copyright (c) 2019 Jonathan Dann

    Forked from https://github.com/jonathan-fielding/yalealarmsystem
    Copyright 2019 Jonathan Fielding, Jack Mellor & Adam Green

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
*/
Object.defineProperty(exports, "__esModule", { value: true });
const NodeFetch = require("node-fetch");
const Decoders_1 = require("./Decoders");
const Model_1 = require("./Model");
const Logger_1 = require("./Logger");
const await_lock_1 = require("await-lock");
const Lock_1 = require("./Lock");
var API;
(function (API) {
    const yaleAuthToken = 'VnVWWDZYVjlXSUNzVHJhcUVpdVNCUHBwZ3ZPakxUeXNsRU1LUHBjdTpkd3RPbE15WEtENUJ5ZW1GWHV0am55eGhrc0U3V0ZFY2p0dFcyOXRaSWNuWHlSWHFsWVBEZ1BSZE1xczF4R3VwVTlxa1o4UE5ubGlQanY5Z2hBZFFtMHpsM0h4V3dlS0ZBcGZzakpMcW1GMm1HR1lXRlpad01MRkw3MGR0bmNndQ==';
    function url(path) {
        return `https://mob.yalehomesystem.co.uk/yapi/${path}/`;
    }
    async function authenticate(username, password) {
        return await NodeFetch.default(url("o/token"), {
            method: 'POST',
            body: `grant_type=password&username=${encodeURIComponent(username)}&password=${encodeURIComponent(password)}`,
            headers: {
                Authorization: `Basic ${yaleAuthToken}`,
                'Content-Type': 'application/x-www-form-urlencoded ; charset=utf-8',
            },
        });
    }
    API.authenticate = authenticate;
    async function getServices(accessToken) {
        return await NodeFetch.default(url("services"), {
            method: 'GET',
            headers: {
                Authorization: `Bearer ${accessToken}`,
            },
        });
    }
    API.getServices = getServices;
    async function getMode(accessToken) {
        return await NodeFetch.default(url("api/panel/mode"), {
            method: 'GET',
            headers: {
                Authorization: `Bearer ${accessToken}`,
            },
        });
    }
    API.getMode = getMode;
    async function setMode(accessToken, mode) {
        return await NodeFetch.default(url("api/panel/mode"), {
            method: 'POST',
            body: `area=1&mode=${mode}`,
            headers: {
                Authorization: `Bearer ${accessToken}`,
                'Content-Type': 'application/x-www-form-urlencoded ; charset=utf-8',
            },
        });
    }
    API.setMode = setMode;
    async function getDevices(accessToken) {
        return await NodeFetch.default(url("api/panel/device_status"), {
            method: 'GET',
            headers: {
                Authorization: `Bearer ${accessToken}`,
            },
        });
    }
    API.getDevices = getDevices;
})(API || (API = {}));
async function authenticate(username, password) {
    let response = await API.authenticate(username, password);
    return await processResponse(response, Decoders_1.JSONDecoders.accessTokenDecoder, (accessToken) => {
        let expiration = new Date();
        expiration.setSeconds(expiration.getSeconds() + accessToken.expiration);
        return {
            token: accessToken.token,
            expiration: expiration,
        };
    });
}
async function processResponse(response, decoder, success) {
    if (response.status === 200) {
        let decoded = decoder.decodeAny(await response.json());
        return success(decoded);
    }
    else if (response.status >= 400 && response.size < 500) {
        let error = Decoders_1.JSONDecoders.errorDecoder.decodeAny(await response.json());
        if (error.description) {
            throw new Error(`HTTP ${response.status} "${error.error}", "${error.description}"`);
        }
        else {
            throw new Error(`HTTP ${response.status} "${error.error}", No description given.`);
        }
    }
    throw new Error(`Unhandled HTTP status code: ${response.status}`);
}
async function getMode(accessToken) {
    let response = await API.getMode(accessToken.token);
    return processResponse(response, Decoders_1.JSONDecoders.panelGetDecoder, (mode) => {
        switch (mode[0].mode) {
            case 'arm':
                return "arm";
            case 'home':
                return "home";
            case 'disarm':
                return "disarm";
            default:
                throw new Error('Something went wrong.');
        }
    });
}
exports.getMode = getMode;
async function getServices(accessToken) {
    let response = await API.getServices(accessToken.token);
    return processResponse(response, Decoders_1.JSONDecoders.servicesDecoder, servicesJSON => {
        return servicesJSON;
    });
}
function parseContactSensorState(value) {
    switch (value) {
        case 'device_status.dc_close':
            return Model_1.ContactSensor.State.Closed;
        case 'device_status.dc_open':
            return Model_1.ContactSensor.State.Open;
    }
}
function parseMotionSensorState(value) {
    switch (value) {
        case 'device_status.pir_triggered':
            return Model_1.MotionSensor.State.Triggered;
        default:
            return Model_1.MotionSensor.State.None;
    }
}
function deviceToSensor(value) {
    switch (value.type) {
        case 'device_type.door_contact':
            let state = parseContactSensorState(value.status);
            if (state !== undefined) {
                return new Model_1.ContactSensor(value.identifier, value.name, state);
            }
            else {
                return undefined;
            }
        case 'device_type.pir':
            return new Model_1.MotionSensor(value.identifier, value.name, parseMotionSensorState(value.status));
        default:
            return undefined;
    }
}
const isPresent = (value) => value != null;
async function getSensors(accessToken) {
    let response = await API.getDevices(accessToken.token);
    return processResponse(response, Decoders_1.JSONDecoders.devicesDecoder, (devices) => {
        return devices.map(deviceToSensor).filter(isPresent);
    });
}
async function setMode(accessToken, mode) {
    let response = await API.setMode(accessToken.token, mode);
    return processResponse(response, Decoders_1.JSONDecoders.panelSetDecoder, (value) => {
        if (value.acknowledgement === 'OK') {
            return mode;
        }
        else {
            throw new Error('Something went wrong.');
        }
    });
}
class Yale {
    constructor(_username, _password, _log = new Logger_1.Logger()) {
        this._username = _username;
        this._password = _password;
        this._log = _log;
        this._contactSensors = {};
        this._motionSensors = {};
        this._lock = new await_lock_1.default();
    }
    async _updatePanel(accessToken) {
        const identifier = await (async () => {
            if (this._panel === undefined) {
                const services = await getServices(accessToken);
                return services.panel;
            }
            else {
                return this._panel.identifier;
            }
        })();
        const panelState = await getMode(accessToken);
        this._panel = new Model_1.Panel(identifier, 'Panel', panelState);
        return this._panel;
    }
    async _updateSensors(accessToken) {
        const sensors = await getSensors(accessToken);
        this._contactSensors = sensors.reduce((map, sensor) => {
            if (sensor instanceof Model_1.ContactSensor) {
                map[sensor.identifier] = sensor;
            }
            return map;
        }, {});
        this._motionSensors = sensors.reduce((map, sensor) => {
            if (sensor instanceof Model_1.MotionSensor) {
                map[sensor.identifier] = sensor;
            }
            return map;
        }, {});
    }
    async update() {
        return await Lock_1.default(this._lock, async () => {
            const accessToken = await authenticate(this._username, this._password);
            await this._updatePanel(accessToken);
            await this._updateSensors(accessToken);
        });
    }
    async panel() {
        return await Lock_1.default(this._lock, async () => {
            return this._panel;
        });
    }
    async motionSensors() {
        return await Lock_1.default(this._lock, async () => {
            return this._motionSensors;
        });
    }
    async contactSensors() {
        return await Lock_1.default(this._lock, async () => {
            return this._contactSensors;
        });
    }
    async getPanelState() {
        return await Lock_1.default(this._lock, async () => {
            const accessToken = await authenticate(this._username, this._password);
            return (await this._updatePanel(accessToken)).state;
        });
    }
    async setPanelState(targetState) {
        return await Lock_1.default(this._lock, async () => {
            const accessToken = await authenticate(this._username, this._password);
            const panelState = await setMode(accessToken, targetState);
            if (this._panel === undefined) {
                return (await this._updatePanel(accessToken)).state;
            }
            else {
                this._panel = new Model_1.Panel(this._panel.identifier, this._panel.name, panelState);
                return panelState;
            }
        });
    }
    async updateMotionSensor(sensor) {
        return await Lock_1.default(this._lock, async () => {
            const accessToken = await authenticate(this._username, this._password);
            await this._updateSensors(accessToken);
            return this._motionSensors[sensor.identifier];
        });
    }
    async updateContactSensor(sensor) {
        return await Lock_1.default(this._lock, async () => {
            const accessToken = await authenticate(this._username, this._password);
            await this._updateSensors(accessToken);
            return this._contactSensors[sensor.identifier];
        });
    }
}
exports.Yale = Yale;
//# sourceMappingURL=Yale.js.map